[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Tick History client library for Python

[![API Version](https://img.shields.io/badge/api-v1.1.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetTickHistory)](https://pypi.org/project/fds.sdk.FactSetTickHistory/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

TickHistory provides dynamic access to historical tick data for a specific security for specific dates or date range.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.1.0
- SDK version: 0.22.13
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetTickHistory==0.22.13
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetTickHistory==0.22.13
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetTickHistory
from fds.sdk.FactSetTickHistory.api import get_files_api
from fds.sdk.FactSetTickHistory.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetTickHistory.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetTickHistory.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetTickHistory.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = get_files_api.GetFilesApi(api_client)
    request_id = "requestId_example" # str | RequestId returned by request-files endpoint to poll and collect results of the query
    pagination_limit = 20 # int | Specifies the maximum number of results to return per response page (optional) if omitted the server will use the default value of 20
    pagination_offset = 0 # int | Specifies the starting point for pagination. This parameter is used to identify the beginning of next set of results (optional) if omitted the server will use the default value of 0

    try:
        # Returns the files from tickhistory endpoint for the specified date range
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.v1_get_files_get(request_id, pagination_limit=pagination_limit, pagination_offset=pagination_offset)

        pprint(api_response)
    except fds.sdk.FactSetTickHistory.ApiException as e:
        print("Exception when calling GetFilesApi->v1_get_files_get: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns the files from tickhistory endpoint for the specified date range
    #     api_response, http_status_code, response_headers = api_instance.v1_get_files_get_with_http_info(request_id, pagination_limit=pagination_limit, pagination_offset=pagination_offset)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetTickHistory.ApiException as e:
    #     print("Exception when calling GetFilesApi->v1_get_files_get: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns the files from tickhistory endpoint for the specified date range
    #     async_result = api_instance.v1_get_files_get_async(request_id, pagination_limit=pagination_limit, pagination_offset=pagination_offset)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetTickHistory.ApiException as e:
    #     print("Exception when calling GetFilesApi->v1_get_files_get: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns the files from tickhistory endpoint for the specified date range
    #     async_result = api_instance.v1_get_files_get_with_http_info_async(request_id, pagination_limit=pagination_limit, pagination_offset=pagination_offset)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetTickHistory.ApiException as e:
    #     print("Exception when calling GetFilesApi->v1_get_files_get: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetTickHistory

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetTickHistory.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetTickHistory

configuration = fds.sdk.FactSetTickHistory.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetTickHistory

configuration = fds.sdk.FactSetTickHistory.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetTickHistory

configuration = fds.sdk.FactSetTickHistory.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api-sandbox.factset.com/bulk-documents/tickhistory*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*GetFilesApi* | [**v1_get_files_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/GetFilesApi.md#v1_get_files_get) | **GET** /v1/get-files | Returns the files from tickhistory endpoint for the specified date range
*RequestFilesApi* | [**v1_request_files_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/RequestFilesApi.md#v1_request_files_get) | **GET** /v1/request-files | Returns the requestId and status


## Documentation For Models

 - [Data](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/Data.md)
 - [GetFilesErrorDetail](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/GetFilesErrorDetail.md)
 - [GetFilesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/GetFilesResponse.md)
 - [GetFilesStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/GetFilesStatus.md)
 - [Meta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/Meta.md)
 - [Pagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/Pagination.md)
 - [Partial](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/Partial.md)
 - [RequestFileErrorDetails](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/RequestFileErrorDetails.md)
 - [RequestFilesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/RequestFilesResponse.md)
 - [RequestFilestatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v1/docs/RequestFilestatus.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetTickHistory.apis and fds.sdk.FactSetTickHistory.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetTickHistory.api.default_api import DefaultApi`
- `from fds.sdk.FactSetTickHistory.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetTickHistory
from fds.sdk.FactSetTickHistory.apis import *
from fds.sdk.FactSetTickHistory.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

